/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizsaleorder.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.biz.modular.bizsaleorder.entity.BizSaleOrder;
import vip.xiaonuo.biz.modular.bizsaleorder.param.BizSaleOrderAddParam;
import vip.xiaonuo.biz.modular.bizsaleorder.param.BizSaleOrderEditParam;
import vip.xiaonuo.biz.modular.bizsaleorder.param.BizSaleOrderIdParam;
import vip.xiaonuo.biz.modular.bizsaleorder.param.BizSaleOrderPageParam;
import vip.xiaonuo.biz.modular.bizsaleorder.service.BizSaleOrderService;

import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 销售订单控制器
 *
 * @author fanzherong
 * @date  2025/04/10 09:31
 */
@Tag(name = "销售订单控制器")
@RestController
@Validated
public class BizSaleOrderController {

    @Resource
    private BizSaleOrderService bizSaleOrderService;

    /**
     * 获取销售订单分页
     *
     * @author fanzherong
     * @date  2025/04/10 09:31
     */
    @Operation(summary = "获取销售订单分页")
    @SaCheckPermission("/biz/bizsaleorder/page")
    @GetMapping("/biz/bizsaleorder/page")
    public CommonResult<Page<BizSaleOrder>> page(BizSaleOrderPageParam bizSaleOrderPageParam) {
        return CommonResult.data(bizSaleOrderService.page(bizSaleOrderPageParam));
    }

    /**
     * 添加销售订单
     *
     * @author fanzherong
     * @date  2025/04/10 09:31
     */
    @Operation(summary = "添加销售订单")
    @CommonLog("添加销售订单")
    @PostMapping("/biz/bizsaleorder/add")
    public CommonResult<String> add(@RequestBody @Valid BizSaleOrderAddParam bizSaleOrderAddParam) {
        bizSaleOrderService.add(bizSaleOrderAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑销售订单
     *
     * @author fanzherong
     * @date  2025/04/10 09:31
     */
    @Operation(summary = "编辑销售订单")
    @CommonLog("编辑销售订单")
    @PostMapping("/biz/bizsaleorder/edit")
    public CommonResult<String> edit(@RequestBody @Valid BizSaleOrderEditParam bizSaleOrderEditParam) {
        bizSaleOrderService.edit(bizSaleOrderEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除销售订单
     *
     * @author fanzherong
     * @date  2025/04/10 09:31
     */
    @Operation(summary = "删除销售订单")
    @CommonLog("删除销售订单")
    @PostMapping("/biz/bizsaleorder/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   List<BizSaleOrderIdParam> bizSaleOrderIdParamList) {
        bizSaleOrderService.delete(bizSaleOrderIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取销售订单详情
     *
     * @author fanzherong
     * @date  2025/04/10 09:31
     */
    @Operation(summary = "获取销售订单详情")
    @GetMapping("/biz/bizsaleorder/detail")
    public CommonResult<BizSaleOrder> detail(@Valid BizSaleOrderIdParam bizSaleOrderIdParam) {
        return CommonResult.data(bizSaleOrderService.detail(bizSaleOrderIdParam));
    }


    /**
     * 根据客户id查询销售订单信息
     *
     * @author fanzherong
     * @date  2025/04/10 09:31
     */
    @Operation(summary = "根据客户id查询销售订单信息")
    @GetMapping("/biz/bizsaleorder/queryByCustomerId")
    public CommonResult<List<BizSaleOrder>> queryByCustomerId(@Valid BizSaleOrderIdParam bizSaleOrderIdParam) {
        return CommonResult.data(bizSaleOrderService.queryByCustomerId(bizSaleOrderIdParam));
    }

    /**
     * 详情（重量转换）
     *
     * @author fanzherong
     * @date  2025/04/10 09:31
     */
    @Operation(summary = "详情（重量转换）")
    @GetMapping("/biz/bizsaleorder/detailById")
    public CommonResult<BizSaleOrder> detailById(@Valid BizSaleOrderIdParam bizSaleOrderIdParam) {
        return CommonResult.data(bizSaleOrderService.detailById(bizSaleOrderIdParam));
    }
}
